import QtQuick 2.4
import QtQuick.Controls 1.3
import QtQuick.Window 2.2

ApplicationWindow {
    title: qsTr("Hello World")
    width: 640
    height: 480
    visible: true

    Component.onCompleted: {
       console.log("onCompleted")
       animation.start()
       console.log("animation started")
    }

    SequentialAnimation {
       id: animation
       PauseAnimation { duration: 1000 }
       PropertyAction { target: textItem; property: "text"; value: "Some text" }

       // 1 - Should work, but doesn't
       //PropertyAction { target: textItem; property: "x"; value: 0.5 * (rect.width - textItem.paintedWidth) }

       // 2 - Just to generate some debug output: dummyComputeTextWidth() called three times before the component and twice when animation is started?
       //PropertyAction { target: textItem; property: "x"; value: 0.5 * (rect.width - textItem.paintedWidth + textItem.dummyComputeTextWidth()) }

       // 3 - With hardcoded offset - text is centered, but hardcoding is not an option.
       //PropertyAction { target: textItem; property: "x"; value: 0.5 * (rect.width - textItem.magicComputeTextWidth()) }

       // 4 - Fails, but you can see that paintedWidth is already set correctly, however the return value has no effect
       PropertyAction { target: textItem; property: "x"; value: 0.5 * (rect.width - textItem.paintedComputeTextWidth()) }

       // 5 - paintedHeight does not cause any problems
       PropertyAction { target: textItem; property: "y"; value: 0.5 * (rect.height - textItem.paintedHeight) }

       // 6 - a function call that returns paintedHeight also works. It is also called multiple five times, though.
       //PropertyAction { target: textItem; property: "y"; value: 0.5 * (rect.height - textItem.computeTextHeight()) }
    }

    Rectangle {
       id: rect
       anchors.fill: parent


       Rectangle {
          width: parent.width
          height: 1
          anchors.verticalCenter: parent.verticalCenter
          color: "darkgrey"
       }

       Rectangle {
          width: 1
          height: parent.height
          anchors.horizontalCenter: parent.horizontalCenter
          color: "darkgrey"
       }

       Text {
          id: textItem
          font.pixelSize: 60

          function dummyComputeTextWidth() {
             console.log("dummy: " + text + " " + rect.width + " " + paintedWidth)
             return 0
          }

          function magicComputeTextWidth() {
             console.log("magic: " + text + " " + rect.width + " " + paintedWidth)
             return 168.140625
          }

          function paintedComputeTextWidth() {
             console.log("painted: " + text + " " + rect.width + " " + paintedWidth)
             return paintedWidth
          }

          function computeTextHeight() {
             console.log("height: " + text + " " + rect.height + " " + paintedHeight)
             return paintedHeight
          }
       }
    }

    MouseArea {
       anchors.fill: parent
       onClicked: console.log("textItem.x " + textItem.x)
    }
}
